ALTER TABLE ka_dokumente_tbl
 ADD COLUMN benutzer_id int4 REFERENCES ka_benutzer_tbl(id) ON DELETE CASCADE,
 ADD CONSTRAINT ka_dokumente_tbl_master_benutzer
          CHECK ( master_id IS NULL OR benutzer_id IS NULL );


CREATE OR REPLACE FUNCTION create_fall_dokument_ds(mid integer, nuuid char(36), filename varchar) RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
        did integer;
        rid integer;
BEGIN
    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    SELECT INTO did id from ka_dokumente_tbl WHERE name = filename AND mid = master_id;
    IF NOT FOUND THEN
        INSERT INTO ka_dokumente_tbl(master_id, benutzer_id)
             VALUES (mid, NULL);
        SELECT INTO rid currval('ka_dokumente_tbl_id_seq');
    ELSE
        RETURN did;
    END IF;

    IF nuuid IS NOT NULL THEN
        UPDATE ka_dokumente_tbl
        SET    uuid_id = nuuid
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('ka_dokumente_tbl', rid);
    END IF;
    RETURN rid;
END;
$$ ;

REVOKE ALL ON FUNCTION create_fall_dokument_ds(integer, char(36), varchar) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_fall_dokument_ds( integer, char(36), varchar) TO GROUP :cm_group;


CREATE OR REPLACE FUNCTION create_global_dokument_ds(filename varchar) RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
    did integer;
BEGIN
    SELECT INTO did id from ka_dokumente_tbl
     WHERE name = filename
       AND master_id IS NULL
       AND benutzer_id IS NULL;

    IF  NOT FOUND THEN
        INSERT INTO ka_dokumente_tbl (master_id, benutzer_id)
             VALUES (NULL, NULL);
        RETURN currval('ka_dokumente_tbl_id_seq');
    ELSE
        RETURN did;
    END IF;
END;
$$ ;

REVOKE ALL ON FUNCTION create_global_dokument_ds(varchar) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_global_dokument_ds(varchar) TO GROUP :admin_group;


CREATE OR REPLACE FUNCTION create_benutzer_dokument_ds(filename varchar) RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
    did integer;
    user_id integer;
BEGIN
    user_id := getuserid();
    SELECT INTO did id FROM ka_dokumente_tbl
     WHERE name = filename
       AND benutzer_id = user_id;

    IF  NOT FOUND THEN
        INSERT INTO ka_dokumente_tbl (master_id, benutzer_id)
             VALUES (NULL, user_id);
        RETURN currval('ka_dokumente_tbl_id_seq');
    ELSE
        RETURN did;
    END IF;
END;
$$ ;

REVOKE ALL ON FUNCTION create_benutzer_dokument_ds(varchar) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_benutzer_dokument_ds(varchar) TO GROUP :cm_group;


CREATE OR REPLACE FUNCTION delete_benutzer_dokument_ds(did integer) RETURNS void
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$
DECLARE
    user_id integer;
BEGIN
    SELECT INTO user_id benutzer_id FROM ka_dokumente_tbl WHERE id = did;
    IF NOT user_id = getuserid() THEN
        RAISE EXCEPTION 'Current user not owner of user document';
    END IF;
    DELETE FROM ka_dokumente_tbl
     WHERE id = did
       AND benutzer_id = getuserid();
END;
$$ ;

REVOKE ALL ON FUNCTION delete_benutzer_dokument_ds(did integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION delete_benutzer_dokument_ds( integer )
            TO GROUP :cm_group;
