BEGIN;

-- Set role
SET ROLE :db_adm;

--
-- Add field inst_aktenzeichen to rg_institution_tbl
--

-- History
INSERT INTO db_history (beschreibung, zeitpunkt)
VALUES ('1.3.1.1/01_aktenzeichen.sql', current_timestamp);

ALTER TABLE master_tbl DISABLE TRIGGER USER;
ALTER TABLE rg_institution_tbl DISABLE TRIGGER USER;
ALTER TABLE rg_institution_tbl ADD COLUMN inst_aktenzeichen VARCHAR(60);

-- Drop and recreate changed views.
DROP VIEW IF EXISTS rg_institution_tbl_view CASCADE;
DROP VIEW IF EXISTS rg_institution_tbl_anonym_view CASCADE;

CREATE OR REPLACE VIEW rg_institution_tbl_view
AS SELECT 
     rg_institution_tbl.id AS id,
     rg_institution_tbl.master_id AS master_id,
     rg_institution_tbl.uuid_id AS uuid_id,
     rg_institution_tbl.inst_art AS inst_art,
     rg_institution_tbl.inst_betreuende_stelle AS inst_betreuende_stelle,
     rg_institution_tbl.inst_aktenzeichen AS inst_aktenzeichen,
     rg_institution_tbl.inst_strasse AS inst_strasse,
     rg_institution_tbl.inst_strassenr AS inst_strassenr,
     rg_institution_tbl.inst_plz AS inst_plz,
     rg_institution_tbl.inst_ort AS inst_ort,
     rg_institution_tbl.inst_person_name AS inst_person_name,
     rg_institution_tbl.inst_person_vorname AS inst_person_vorname,
     rg_institution_tbl.inst_vorwahl AS inst_vorwahl,
     rg_institution_tbl.inst_telefon AS inst_telefon,
     rg_institution_tbl.inst_email AS inst_email
FROM rg_institution_tbl
    INNER JOIN master_tbl
    ON         rg_institution_tbl.master_id = master_tbl.id
WHERE
    master_tbl.bearbeiter_id = getuserid()
    OR master_tbl.id IN (
        SELECT nm.master_id from nm_benutzer_master_tbl nm
        WHERE nm.benutzer_id IN (
            SELECT nm2.gruppe_id from nm_benutzer_gruppe_tbl nm2
            WHERE nm2.benutzer_id = getuserid()
        )
    )
    OR pg_has_role('admin','MEMBER');

CREATE OR REPLACE RULE rg_institution_tbl_update_rule AS ON UPDATE TO rg_institution_tbl_view
DO INSTEAD UPDATE rg_institution_tbl SET
    inst_art = NEW.inst_art,
    inst_betreuende_stelle = NEW.inst_betreuende_stelle,
    inst_aktenzeichen = NEW.inst_aktenzeichen,
    inst_strasse = NEW.inst_strasse,
    inst_strassenr = NEW.inst_strassenr,
    inst_plz = NEW.inst_plz,
    inst_ort = NEW.inst_ort,
    inst_person_name = NEW.inst_person_name,
    inst_person_vorname = NEW.inst_person_vorname,
    inst_vorwahl = NEW.inst_vorwahl,
    inst_telefon = NEW.inst_telefon,
    inst_email = NEW.inst_email
WHERE id = NEW.id;


CREATE OR REPLACE RULE rg_institution_tbl_delete_rule AS ON DELETE TO rg_institution_tbl_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE rg_institution_tbl_insert_rule AS ON INSERT TO rg_institution_tbl_view
    DO INSTEAD NOTHING;

GRANT SELECT ON rg_institution_tbl_view TO GROUP :cm_group;
GRANT UPDATE ON rg_institution_tbl_view TO GROUP :cm_group;
GRANT SELECT ON rg_institution_tbl_view TO GROUP :admin_group;

CREATE OR REPLACE VIEW rg_institution_tbl_anonym_view
AS SELECT
    id,
    master_id,
    uuid_id,
    inst_art,
    NULL::VARCHAR(60) AS inst_betreuende_stelle,
    NULL::VARCHAR(60) AS inst_aktenzeichen,
    NULL::VARCHAR(60) AS inst_strasse,
    NULL::VARCHAR(60) AS inst_strassenr,
    NULL::VARCHAR(5) AS inst_plz,
    NULL::VARCHAR(60) AS inst_ort,
    NULL::VARCHAR(60) AS inst_person_name,
    NULL::VARCHAR(60) AS inst_person_vorname,
    NULL::VARCHAR(60) AS inst_vorwahl,
    NULL::VARCHAR(60) AS inst_telefon,
    NULL::VARCHAR(60) AS inst_email FROM rg_institution_tbl_view;

CREATE OR REPLACE RULE rg_institution_tbl_anonym_delete_rule AS ON DELETE TO rg_institution_tbl_anonym_view
    DO INSTEAD NOTHING;

CREATE OR REPLACE RULE rg_institution_tbl_anonym_insert_rule AS ON INSERT TO rg_institution_tbl_anonym_view
    DO INSTEAD NOTHING;

GRANT SELECT ON rg_institution_tbl_anonym_view TO GROUP :cm_group;
GRANT SELECT ON rg_institution_tbl_anonym_view TO GROUP :admin_group;

DROP FUNCTION IF EXISTS anonymize_case(int);
CREATE OR REPLACE FUNCTION anonymize_case(mid int)
RETURNS void AS
$$
BEGIN
    PERFORM anonymize_uuid(mid);
    -- anonymize master table
    UPDATE master_tbl_view SET
        jmd_name = NULL,
        nennung_einrichtung_cm = NULL,
        name = NULL,
        vorname = NULL,
        geburtsname = NULL,
        name_vor_einbuergerung = NULL,
        geburtsland_nennung = NULL,
        geburtsort = NULL,
        fuehrerschein = NULL,
        weitere_angaben_kinder = NULL,
        staatsangehoerigkeit_nennung = NULL,
        staatsangehoerigkeit_weitere_nennung = NULL,
        muttersprache_nennung = NULL,
        sprache_zuhause_nennung = NULL,
        wohnsituation_konkretisierung = NULL,
        bezugsperson_weitere_angaben = NULL,
        soz_engagement = NULL,
        hobbies = NULL,
        anmerkungen_finanzsituation = NULL,
        anmerkungen_gensundsituation = NULL,
        aktuelle_situation_nennung = NULL,
        aktueller_beruf = NULL,
        aktuelle_massnahme_art_nennung = NULL,
        aktuelle_massnahme = NULL,
        aktuelle_taetigkeit = NULL,
        hoechster_schulabschluss_nennung = NULL,
        hoechster_schulabschluss_ausland = NULL,
        weitere_angaben_schulabschluss = NULL,
        hoechster_berufsabschluss_nennung = NULL,
        hoechster_berufsabschluss_ausland = NULL,
        weitere_angaben_berufsabschluss = NULL,
        gruende_abbruch_cm_nennung = NULL,
        ende_inst_einbindung_nennung = NULL,
        verbleib_jugendliche_sonstiges = NULL
    WHERE id = mid;

    -- anonymize repeat groups
    UPDATE rg_kontaktdaten_tbl_view SET
        adresse_strasse = NULL,
        adresse_strassenr = NULL,
        adresse_plz = shorten_value(adresse_plz, 3),
        adresse_ort = NULL,
        vorwahl = NULL,
        telefon = NULL,
        mobilfunknummer = NULL,
        email = NULL
    WHERE master_id = mid;

    UPDATE rg_kontaktdaten_sorgeberechtigte_tbl_view SET
        name_sorge = NULL,
        vorname_sorge = NULL,
        vorwahl_sorge = NULL,
        telefon_sorge = NULL,
        mobilfunknummer_sorge = NULL,
        email_sorge = NULL
    WHERE master_id = mid;

    UPDATE rg_institution_tbl_view SET
        inst_betreuende_stelle = NULL,
        inst_aktenzeichen = NULL,
        inst_strasse = NULL,
        inst_strassenr = NULL,
        inst_plz = NULL,
        inst_ort = NULL,
        inst_person_name = NULL,
        inst_person_vorname = NULL,
        inst_vorwahl = NULL,
        inst_telefon = NULL,
        inst_email = NULL
    WHERE master_id = mid;

    UPDATE rg_biographie2_tbl_view SET
        biographie_anmerkung = NULL
    WHERE master_id = mid;

    UPDATE rg_weitere_sprache_tbl_view SET
        kf_erg_weiteresprache_nennung_sonstiges = NULL
    WHERE master_id = mid;

    UPDATE rg_ziele_tbl_view SET
        fp_ziel_sonstiges = NULL,
        fp_ziel_beschreibung = NULL,
        fp_leistung_teilnehmer = NULL,
        fp_leistung_jmd = NULL,
        fp_uziel1 = NULL,
        ua_uziel1_art = NULL,
        fp_externe_einrichtung_1_name = NULL,
        fp_uziel2 = NULL,
        ua_uziel2_art = NULL,
        fp_externe_einrichtung_2_name = NULL,
        fp_uziel3 = NULL,
        ua_uziel3_art = NULL,
        fp_externe_einrichtung_3_name = NULL,
        fp_uziel4 = NULL,
        ua_uziel4_art = NULL,
        fp_externe_einrichtung_4_name = NULL,
        fp_uziel5 = NULL,
        ua_uziel5_art = NULL,
        fp_externe_einrichtung_5_name = NULL
    WHERE master_id = mid;

END;
$$ LANGUAGE plpgsql VOLATILE SECURITY DEFINER;

REVOKE ALL ON FUNCTION anonymize_case(int) FROM public;
GRANT EXECUTE ON FUNCTION anonymize_case(int) TO GROUP :admin_group;


-- Finish, re-enable the triggers
SET ROLE postgres;
ALTER TABLE master_tbl ENABLE TRIGGER ALL;
ALTER TABLE rg_institution_tbl ENABLE TRIGGER ALL;

COMMIT;
