--
-- Issue 3288 
-- Add a new repeat group rg_sprache_zuhause_weitere
--
-- New repeat group table
-- creation

SET ROLE :db_adm;

CREATE TABLE rg_sprache_zuhause_weitere_tbl (
    id SERIAL PRIMARY KEY NOT NULL,
    master_id INTEGER NOT NULL,
    uuid_id CHAR(36),
    sprache_zuhause_weitere INTEGER DEFAULT -1,
    sprache_zuhause_weitere_nennung VARCHAR(60),
    FOREIGN KEY (master_id) REFERENCES master_tbl (id) ON DELETE CASCADE,
    UNIQUE (uuid_id),
    FOREIGN KEY (sprache_zuhause_weitere) REFERENCES muttersprache_tbl (id)
);
-- rgroups changes
-- PLPython is an untrusted language. -> Need to be postgres.
SET ROLE postgres;

-- update the compute_case_structure function as postgres
DROP FUNCTION compute_case_structure(int4);
CREATE OR REPLACE FUNCTION compute_case_structure(case_id int4) RETURNS TEXT
AS $$

class Node(object):

    def __init__(self, name, id=None, children=[]):
        self.name = name
        self.id = id
        self.children = children

    def to_json(self, out):
        out.append('{"name":"%s","id":%d' % (self.name, self.id))
        if self.children:
            out.append(',"children":[')
            first = True
            for child in self.children:
                if first:
                    first = False
                else:
                    out.append(",")
                child.to_json(out)
            out.append("]")
        out.append("}")

    def recursive_build(self, mid, children, parent_name="master"):
        select = ("SELECT id FROM %s WHERE %s_id = %d"
                  % (self.name, parent_name, mid))
        ids = [r['id'] for r in plpy.execute(select)]
        for id in ids:
            sub_children = []
            node = Node(self.name, id, sub_children)
            name = self.name[:-4] # cut off trailing '_tbl'
            for child in self.children:
                child.recursive_build(id, sub_children, name)
            children.append(node)

TREE_STRUCTURE = Node('master_tbl', children = [
    Node('rg_kontaktdaten_tbl'),
    Node('rg_kontaktdaten_sorgeberechtigte_tbl'),
    Node('rg_institution_tbl'),
    Node('rg_kinder_tbl'),
    Node('rg_sprache_zuhause_weitere_tbl'),
    Node('rg_biographie2_tbl'),
    Node('rg_weitere_sprache_tbl'),
    Node('rg_ziele_tbl')
])


children = []
tree = Node(TREE_STRUCTURE.name, case_id, children)
for child in TREE_STRUCTURE.children:
    child.recursive_build(case_id, children)
out = []
tree.to_json(out)
return "".join(out)

$$ LANGUAGE plpythonu EXTERNAL SECURITY DEFINER;
ALTER FUNCTION compute_case_structure(int4) OWNER TO :adm_ka_owner;

-- Continue as db-owner from here
SET ROLE :adm_ka_owner;

-- rgroup changes
-- Set rg rules and trigger
CREATE OR REPLACE FUNCTION rg_sprache_zuhause_weitere_tbl_insert_func() RETURNS TRIGGER AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid NEW.master_id;
    UPDATE case_structure
        SET modified = FALSE, cache = compute_case_structure(master_id)
        WHERE master_id = mid;
    PERFORM set_lastaccess_time(mid);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER rg_sprache_zuhause_weitere_tbl_insert_trigger AFTER INSERT ON rg_sprache_zuhause_weitere_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_sprache_zuhause_weitere_tbl_insert_func();

CREATE OR REPLACE FUNCTION rg_sprache_zuhause_weitere_tbl_update_func() RETURNS TRIGGER AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid NEW.master_id;
    PERFORM set_lastaccess_time(mid);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql EXTERNAL SECURITY DEFINER;

CREATE TRIGGER rg_sprache_zuhause_weitere_tbl_update_trigger AFTER UPDATE ON rg_sprache_zuhause_weitere_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_sprache_zuhause_weitere_tbl_update_func();

CREATE OR REPLACE FUNCTION rg_sprache_zuhause_weitere_tbl_delete_func() RETURNS TRIGGER AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid OLD.master_id;
    UPDATE case_structure
        SET modified = FALSE, cache = compute_case_structure(master_id)
        WHERE master_id = mid;
    PERFORM set_lastaccess_time(mid);
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER rg_sprache_zuhause_weitere_tbl_delete_trigger AFTER DELETE ON rg_sprache_zuhause_weitere_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_sprache_zuhause_weitere_tbl_delete_func();

-- rgroups
-- Set rg create and delete functions
CREATE OR REPLACE FUNCTION create_rg_sprache_zuhause_weitere_tbl(pid integer, nuuid char(36))
RETURNS integer
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$
DECLARE
    rid integer;
    mid integer;
BEGIN
    mid := pid;

    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    INSERT INTO rg_sprache_zuhause_weitere_tbl (master_id) VALUES (pid);

    SELECT INTO rid currval('rg_sprache_zuhause_weitere_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE rg_sprache_zuhause_weitere_tbl
        SET    uuid_id = nuuid
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('rg_sprache_zuhause_weitere_tbl', rid);
        
    END IF;

    RETURN rid;
END;
$$;

CREATE OR REPLACE FUNCTION delete_rg_sprache_zuhause_weitere_tbl(rid integer)
RETURNS void
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid master_id FROM rg_sprache_zuhause_weitere_tbl WHERE id = rid;

    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    DELETE FROM rg_sprache_zuhause_weitere_tbl WHERE id = rid;
END;
$$;
