# -*- coding: utf-8 -*-
# Copyright 2020 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls JMD.
#
# mpuls JMD is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls JMD is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls JMD. If not, see <http://www.gnu.org/licenses/>.
#
import logging

import formencode

from mpulsweb.lib.base import BaseController, render, render_htmlfill, \
     request, c
from mpulsweb.lib.security import checkRole

from jmdcoachesweb.model.stellen import get_stellen, set_stellen
from jmdcoachesweb.lib.validators import StellenForm


log = logging.getLogger(__name__)


class StellenController(BaseController):

    @checkRole("cm")
    def edit(self):
        stellen = StellenForm().from_python(get_stellen())
        return self._render_stellen(defaults=stellen)

    def _render_stellen(self, defaults, errors=None):
        return render_htmlfill(render('stellen/stellen.mako'),
                               defaults=defaults,
                               errors=(errors if errors is not None else {}))

    @checkRole("cm")
    def edit_action(self):
        try:
            form_result = StellenForm().to_python(request.params)
        except formencode.Invalid as error:
            return self._render_stellen(defaults=error.value,
                                        errors=error.error_dict)

        c.success_text = (u"Die Personen und Stellenanteile wurden"
                          " erfolgreich gespeichert")

        set_stellen(personen=form_result["personen"],
                    stellenanteile=form_result["stellenanteile"])
        return self.edit()



# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
