# -*- coding: utf-8 -*-
#
# Copyright 2020 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls_S (CoMPUter-based case fiLeS -
# Deutschlandstipendium).
#
# mpuls_S is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls_S is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls_S. If not, see <http://www.gnu.org/licenses/>.

from mpulsweb.controllers.formularpage import FormularpageController as \
    MpulsFormularpageController, PageRenderer
from formed.instance.tree import IDENTIFIER_SEPARATOR


class FormularpageController(MpulsFormularpageController):

    def _before_set_data_hook(self, case, form_result):
        """Overwrite to reset mitarbeit_netzwerk if network doesn't exist"""
        instance_tree = case.get_formed_instance_tree()
        networks = instance_tree.getAllItemsForName("zielgruppen_netzwerk")
        for network in networks:
            network_id = network.getIdentifier()
            # Note: The following condition matches the expression in
            # the conditional around mitarbeit_netzwerk in the
            # formedtree, so that this code resets mitarbeit_netzwerk
            # exactly when the conditional makes the field uneditable.
            if form_result.get(network_id) not in ("1", "2"):
                parent_ids = network_id.split(IDENTIFIER_SEPARATOR)[1:]
                mitarbeit_id = IDENTIFIER_SEPARATOR.join(["mitarbeit_netzwerk"]
                                                         + parent_ids)
                form_result[mitarbeit_id] = "-1"
