# -*- coding: utf-8 -*-
#
# Copyright 2007, 2008 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls JMD
#
# mpuls JMD is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls JMD is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls JMD. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls JMD has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.
#
# Authors:
# Torsten Irländer <torsten.irlaender@intevation.de>
# Sascha L. Teichmann <teichmann@intevation.de>
#

import logging

import formencode
from pylons.controllers.util import abort

from mpulsweb.model.user import UserListObject
from mpulsweb.lib.base import BaseController, c, render, render_htmlfill, \
    request, h
from mpulsweb.lib.security import checkRole

from jmdweb.lib.validators import PermissionForm, CoachesPermissionForm
from jmdweb.model.struktur import get_struktur_users, set_struktur_users, \
     get_coaches_users, set_coaches_users, coaches_part_available


log = logging.getLogger(__name__)


class JmdstrukturController(BaseController):

    @checkRole(['cm', 'admin'])
    def index(self):
        return render('jmdstruktur/overview.mako')

    @checkRole('admin')
    def permission(self):
        userlist = UserListObject()
        c.userlist = userlist.getUserList(filter=[u.id for u in
                                                  userlist.getAdminList()])
        c.show_coaches_form = coaches_part_available()
        defaults = dict(userid=get_struktur_users(),
                        coaches_userid=get_coaches_users())
        return render_htmlfill(render('jmdstruktur/permission.mako'),
                               defaults=defaults)

    @checkRole('admin')
    def permissionAction(self):
        form_result = PermissionForm().to_python(request.params)
        set_struktur_users(form_result['userid'])
        c.status_msg = h.status_message("Berechtigung erfolgreich gesetzt.")
        return self.permission()

    @checkRole('admin')
    def coachesPermissionAction(self):
        if not coaches_part_available():
            abort(404)
        form_result = CoachesPermissionForm().to_python(request.params)
        set_coaches_users(form_result['coaches_userid'])
        c.status_msg = h.status_message("Berechtigung erfolgreich gesetzt.")
        return self.permission()


# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8:
