# -*- coding: utf-8 -*-
#
# Copyright 2014, 2015 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of MPULS (CoMPUter-based case fiLeS).
#
# MPULS is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# MPULS is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with MPULS. If not, see <http://www.gnu.org/licenses/>.

import logging

from mpulsweb.lib.db import db
from mpulsweb.lib.searchexpr import searchexpr_from_json


log = logging.getLogger(__name__)


LOAD_SAVED_SEARCHES = """
SELECT id, name, formed_version, search FROM saved_search_tbl_view"""


LOAD_SAVED_SEARCH = """
SELECT id, name, formed_version, search FROM saved_search_tbl_view
WHERE id = %(id)s"""


DELETE_SAVED_SEARCH = """
SELECT delete_savedsearch_ds(%(id)s)"""


RENAME_SAVED_SEARCH = """
SELECT rename_savedsearch_ds(%(id)s, %(name)s)"""


NEW_SAVED_SEARCH = """
SELECT create_saved_search_ds(%(name)s, %(formed_version)s, %(search)s)"""


class SavedSearch(object):

    @classmethod
    def get_saved_search_by_id(cls, id1):
        result = None
        with db.cursor() as cur:
            cur.execute(LOAD_SAVED_SEARCH, {'id': id1})
            row = cur.fetchone()
            result = cls(row[0], row[1], row[2], search_json=row[3])
        return result

    @classmethod
    def list_saved_searches(cls):
        result = []
        with db.cursor() as cur:
            cur.execute(LOAD_SAVED_SEARCHES)
            for row in cur.fetchall():
                result.append(cls(row[0], row[1], row[2], search_json=row[3]))
        return result


    def __init__(self, id, name, formed_version, search=None, search_json=None):
        if search is None and search_json is None:
            raise ValueError("At least one of search or search_json must"
                             " be given")

        self.id = id
        self.name = name
        self.formed_version = formed_version
        if search_json is not None:
            search = searchexpr_from_json(search_json)
        self.search_ast = search

    def get_name(self):
        return self.name

    def get_search_json(self):
        return self.search_ast.to_json()

    def get_searchexpr(self):
        return self.search_ast

    def delete(self):
        with db.transaction() as cur:
            cur.execute(DELETE_SAVED_SEARCH, {'id': self.id})

    def rename(self, new_name):
        self.name = new_name
        with db.transaction() as cur:
            cur.execute(RENAME_SAVED_SEARCH,
                        {'id': self.id, 'name': self.get_name()})

    def new(self):
        with db.transaction() as cur:
            cur.execute(NEW_SAVED_SEARCH,
                        {'name': self.name,
                         'formed_version': self.formed_version,
                         'search': self.get_search_json()})
