function getFormFieldElem(fieldname, pseudoclass) {
  if (typeof pseudoclass == "undefined") {
    pseudoclass = ""
  }
  return document.querySelector("#formularpage *[name=\"" + fieldname + "\"]"
                                + pseudoclass)
}


function getFormFieldElemsMeta(prefix) {
  return document.querySelectorAll("#formularpage *[name^=\"" + prefix + ":\"]")
}


function getFormGroupElemsMeta(prefix) {
  return document.querySelectorAll("#formularpage table[id^=\"table-" + prefix + ":\"]")
}


function getHiddenFieldStr(fieldname) {
  var elem = getFormFieldElem(fieldname)
  return elem === null ? null : elem.value
}


function splitIdentifier(identifier) {
  var parts = identifier.split(":")
  if (parts.length === 2) {
    var [name, caseid] = parts
    return [name, null, caseid]
  } else if (parts.length === 3) {
    return parts
  } else {
    console.log("splitIdentifier: unexpected number of parts: " + parts)
    return null
  }
}


function rgIdFromIdentifier(identifier) {
  return splitIdentifier(identifier)[1]
}


function parseDate(datestr) {
  var parts = datestr.split(".")
  if (parts.length === 3) {
    var [sd, sm, sy] = parts
    return new Date(Number.parseInt(sy, 10),
                    Number.parseInt(sm, 10),
                    Number.parseInt(sd, 10))
  }
  return null
}

function getFieldValue(fieldname, caseid, rgid) {
  fieldInfo = form_info.fields[fieldname]

  var fullName = fieldname
  if (fieldInfo.rgs.length > 0) {
    if (rgid !== null) {
      fullName += (rgid !== null ? ":" + rgid : "")
    } else {
      console.log("getFieldValue: no rgid for field in RG: " + fieldname)
    }
  }
  fullName += ":" + caseid

  var converter = fieldInfo.converter
  var elem = getFormFieldElem(fullName, converter === "radio" ? ":checked" : "")
  if (elem !== null) {
    var value = null
    switch (converter) {
    case "int":
    case "radio":
      value = Number.parseInt(elem.value, 10)
      break;
    case "bool":
      value = elem.checked ? 1 : 0
      break;
    case "str":
      // nothing to be done
      value = elem.value
      break;
    case "date":
      value = parseDate(elem.value)
      break;
    default:
      console.log("Unknown converter: " + converter)
    }
    return value
  } else {
    console.log("getFieldValue: cannot find field " + fullName)
  }
}


function isKnownValue(x) {
  switch (typeof x) {
  case "string":
    return x !== "unbekannt"
  case "number":
    return x !== -9999999
  case "object":
    if (x instanceof Date) {
      return +x !== +(new Date(1, 1, 1))
    }
    return true
  default:
    return true
  }
}

function evalExpr(expr, caseid, rgid) {
  var op = expr[0]
  switch (op) {
  case "And":
    return evalExpr(expr[1], caseid, rgid) && evalExpr(expr[2], caseid, rgid)
  case "Or":
    return evalExpr(expr[1], caseid, rgid) || evalExpr(expr[2], caseid, rgid)
  case "EQ":
    var a = evalExpr(expr[1], caseid, rgid)
    var b = evalExpr(expr[2], caseid, rgid)
    // FIXME: this comparison does not work correctly with Date values
    return a === b
  case "NE":
    var a = evalExpr(expr[1], caseid, rgid)
    var b = evalExpr(expr[2], caseid, rgid)
    // FIXME: this comparison does not work correctly with Date values
    return a !== b
  case "GT":
    var a = evalExpr(expr[1], caseid, rgid)
    var b = evalExpr(expr[2], caseid, rgid)
    return a > b
  case "GE":
    var a = evalExpr(expr[1], caseid, rgid)
    var b = evalExpr(expr[2], caseid, rgid)
    return a >= b
  case "LT":
    var a = evalExpr(expr[1], caseid, rgid)
    var b = evalExpr(expr[2], caseid, rgid)
    return a < b
  case "LE":
    var a = evalExpr(expr[1], caseid, rgid)
    var b = evalExpr(expr[2], caseid, rgid)
    return a <= b
  case "CONTAINS":
    var a = evalExpr(expr[1], caseid, rgid)
    var values = evalExpr(expr[2], caseid, rgid)
    return values.indexOf(a) >= 0
  case "NOT":
    var a = evalExpr(expr[1], caseid, rgid)
    return !a
  case "ISSET":
    var a = evalExpr(expr[1], caseid, rgid)
    return a !== null
  case "KNOWN":
    var a = evalExpr(expr[1], caseid, rgid)
    return isKnownValue(a)
  case "Const":
    return expr[1]
  case "List":
    return expr.slice(1).map(function (e) { return evalExpr(e, caseid, rgid) })
  case "Date":
    var day = evalExpr(expr[1], caseid, rgid)
    var month = evalExpr(expr[2], caseid, rgid)
    var year = evalExpr(expr[3], caseid, rgid)
    return new Date(year, month, day)
  case "Field":
    return getFieldValue(expr[1], caseid, rgid)
  default:
    console.log("evalExpr: could not handle expr " + expr)
  }
}


function evaluateConditionals(names, conditionals, caseid, rgid) {
  var total = true
  for (var i = 0; i < names.length; i++) {
    var condInfo = conditionals[names[i]]
    var result = evalExpr(condInfo.expr, caseid, rgid)

    total = total && result
  }
  return total
}


function containingFieldSet(elem) {
  while (elem !== null && elem.tagName !== "FIELDSET") {
    elem = elem.parentElement
  }
  return elem
}


function showHideWarning(elem, doShow) {
  // Determine the name used in the li-element in the warning box that
  // show the warning for elem.
  //
  // For most input types, this is the name of the element. For
  // bool-groups, the warning in the warning box is linked to the
  // containing fieldset element, we need to use the fieldset's id as
  // the warning_elem_name
  var warning_elem_name = elem.name
  var fieldset = containingFieldSet(elem)
  if (fieldset !== null && fieldset.id.startsWith("bool-group-")) {
    warning_elem_name = fieldset.id
  }

  var warning = document.querySelector('li[id^="formed-warning"]'
                                       + '[id$="' + warning_elem_name + '"]')
  if (warning !== null) {
    if (doShow) {
      warning.removeAttribute("style")
    } else {
      warning.style = "display:none;"
    }
    return true
  }
  return false
}

function showHideWarningBox() {
  var box = document.querySelector('div.warning-box')
  if (box !== null) {
    var visibleDescendants = box.querySelectorAll('li:not([style])')
    if (visibleDescendants.length !== 0) {
      box.removeAttribute("style")
    } else {
      box.style = "display:none;"
    }
  }
}


function updateWarningClass(elem, add) {
  if (elem !== null) {
    if (add) {
      if (!elem.classList.contains("error")) {
        elem.classList.add("warning")
      }
    } else {
      elem.classList.remove("warning")
    }
  }
}

function updateFormItemClass(elem) {
  var add = !elem.disabled
  updateWarningClass(elem, add)
  var label = document.querySelector("a[name=\"f_" + elem.id + "\"]")
  updateWarningClass(label, add)
}


function recomputeConditionals() {
  var [pagename, pagergid, caseid] = splitIdentifier(getHiddenFieldStr("page"))
  var conditionals = form_info.pages[pagename].conds

  var affected = form_info.pages[pagename].affected_fields
  for (var fieldName in affected) {
    var fieldConds = affected[fieldName]
    var fieldElems = getFormFieldElemsMeta(fieldName)
    for (var elemIdx = 0; elemIdx < fieldElems.length; elemIdx++) {
      var elem = fieldElems[elemIdx]
      var rgid = rgIdFromIdentifier(elem.name)
      var result = evaluateConditionals(fieldConds, conditionals, caseid, rgid)

      elem.disabled = !result

      var hasWarnings = showHideWarning(elem, result)
      if (hasWarnings) {
        updateFormItemClass(elem)
      }
    }
  }

  affected = form_info.pages[pagename].affected_groups
  for (var groupName in affected) {
    var groupConds = affected[groupName]
    var groupElems = getFormGroupElemsMeta(groupName)
    for (var elemIdx = 0; elemIdx < groupElems.length; elemIdx++) {
      var elem = groupElems[elemIdx]
      var rgid = rgIdFromIdentifier(elem.id)
      var result = evaluateConditionals(groupConds, conditionals, caseid, rgid)

      if (result) {
        elem.classList.remove("invisible_conditional")
      } else {
        elem.classList.add("invisible_conditional")
      }
    }
  }

  showHideWarningBox()
}


function installChangeHandler(elem) {
  elem.addEventListener("change", function(event) {
    var t = event.target;
    recomputeConditionals()
  })
}


window.addEventListener("load", function(event) {
  var page = getHiddenFieldStr("page")
  if (page === null) {
    // don't do anything on pages that aren't forms
    return
  }

  var [pagename, pagergid, caseid] = splitIdentifier(page)
  var sensitive = form_info.pages[pagename].sensitive

  for (var i = 0; i < sensitive.length; i++) {
    var metaName = sensitive[i]
    var elems = getFormFieldElemsMeta(metaName)
    for (var j = 0; j < elems.length; j++) {
      installChangeHandler(elems[j])
    }
  }
});
