## -*- coding: utf-8 -*-
<%!  import mpulsweb.lib.helper.filters as F %>
<%!
   import mpulsweb.lib.searchexpr as S
   import mpulsweb.lib.sqlexpr as E
   from mpulsweb.lib.translation import N_

   match_operators = {
       E.EQ: N_("Equals to"),
       E.LT: N_("Less than"),
       E.LE: N_("Less or equal to"),
       E.GT: N_("Greater than"),
       E.GE: N_("Greater or equal to"),
       E.Startwith: N_("Starts with"),
       E.Substring: N_("Contains"),
       "Between": N_("Between"),
       E.IS_NULL: N_("Unspecified"),
   }
   match_operators_date = {
       E.EQ: N_("On"),
       E.LT: N_("Before"),
       E.LE: N_("Before or on"),
       E.GT: N_("After"),
       E.GE: N_("After or on"),
       "Between": N_("Between"),
       E.IS_NULL: N_("Unspecified"),
   }
%>
<link rel="stylesheet" href="${h.url('/styles/typeahead.css')}">

<script src="${h.url('/js/daterange.js')}"></script>
<script src="${h.url('/js/twitter-typeahead.bundle.js')}"></script>
<script>
$(document).ready(function() {

    /*
     * Autocompleter
     */
    var search = new Bloodhound({
        //datumTokenizer: Bloodhound.tokenizers.obj.whitespace('description'),
        datumTokenizer: function(d){
            // Naive tokenizer so that not only hits at the beginning of the
            // word are matched, see
            // * http://stackoverflow.com/questions/23448193/
            // * https://github.com/twitter/typeahead.js/issues/96
            var tokens = [];
            var stringSize = d.description.length;

            // multiple combinations for every available size
            // dog => d, o, g, do, og, dog
            for (var size = 1; size <= stringSize; size++){
                for (var i = 0; i+size <= stringSize; i++){
                    tokens.push(d.description.substr(i, size));
                }
            }
            return tokens;
        },
        queryTokenizer: Bloodhound.tokenizers.whitespace,
        limit: 10,
        local: ${c.formed_fields|n},
    });

    search.initialize();

    $('.typeahead').typeahead(null, {
        name: 'formed-search',
        source: search.ttAdapter(),
        displayKey: 'description',
    });
});
</script>
<%def name="maybeattr(attrname, cond)">
% if cond:
${attrname}
% endif
</%def>

## Render an option for a choice field
<%def name="option(value, desc, selected)">
<option value="${value}" ${maybeattr("selected", selected)}>${desc}</option>
</%def>

## Render a generic choice field
<%def name="render_search_choice(expr, selected_values, desc)">
<% key = expr.form_key() %>
% if len(desc.extras) > 5:
  <select multiple size="5" name="${key}_value">
    % for pk, value in desc.extras:
    ${option(pk, value,
             selected_values is not None and pk in selected_values)}
    % endfor
  </select>
% else:
  % for pk, value in desc.extras:
  <input name="${key}_value" type="checkbox" value="${pk}"
         ${maybeattr("checked",
                     selected_values is not None and pk in selected_values)}
         >${value}<br>
  % endfor
% endif
</%def>

<%def name="render_search_radio(expr, selected_value, desc)">
<% key = expr.form_key() %>
% for pk, value in desc.extras:
<input name="${key}_value" type="radio" value="${pk}"
     ${maybeattr("checked", pk == selected_value)}
     >${value}<br>
% endfor
</%def>

<%def name="render_search_bool(expr, selected_value, desc)">
<% key = expr.form_key() %>
% for pk, value in [(0, _("Set")), (-1,_("Not set"))]:
<input name="${key}_value" type="radio" value="${pk}"
     ${maybeattr("checked", pk == selected_value)}
     >${value}<br>
% endfor
</%def>

<%def name="render_match_operator(op, selected, type)">
<%
   opname = op
   if not isinstance(opname, basestring):
       opname = op.__name__
%>
%if type == "date":
    ## Special case handling for date because they
    ## should be named differently.
    ${option(opname, _(match_operators_date[op]), selected)}
%else:
    ${option(opname, _(match_operators[op]), selected)}
%endif
</%def>


<%def name="render_search_entry(expr, desc)">
<%
    extra_css_classes = []
    extra_range_classes = ["input-group"]
    key = expr.form_key() 
    if desc.type == "date":
        operators = [E.EQ, E.LT, E.LE, E.GT, E.GE, "Between", E.IS_NULL]
        extra_css_classes.append("mpuls-datepicker")
        extra_range_classes.append("input-daterange")
    elif desc.type == "text":
        operators = [E.Startwith, E.Substring, E.EQ]
    elif desc.type == "int":
        operators = [E.EQ, E.LT, E.LE, E.GT, E.GE, E.IS_NULL]

    if isinstance(expr, S.Between):
        current_op = "Between"
        value_low = h.format_object(expr.low)
        value_high = h.format_object(expr.high)
        value = value_low + "-" + value_high
    else:
        current_op = expr.op
        value = h.format_object(expr.value)
        value_low = ""
        value_high = ""

    if desc.error is not None:
        value = desc.error.value
        extra_css_classes.extend(["error_box", "error"])

%>
<select name="${key}_type" style="width:30%;">
  % for op in operators:
  ${render_match_operator(op, op == current_op, desc.type)}
  % endfor
</select>
<input type="text" name="${key}_value" value="${value}"
       class="${" ".join(extra_css_classes)}" style="width:65%;">
%if "Between" in operators:
<span style="display: none;" class="${" ".join(extra_range_classes)}">
  <input type="text" name="${key}_value_low" value="${value_low}"
         class="form-control" style="width:25%">
  <span>bis</span>
  <input type="text" name="${key}_value_high" value="${value_high}"
         class="form-control" style="width:25%">
</span>
%endif
</%def>


<%def name="render_search_case_id(expr, desc)">
<%
  key = expr.form_key()
%>
<input type="text" name="${key}_value"
       % if desc.error is not None:
       value="${desc.error.value}"
       class="error_box error"
       % else:
       value="${h.format_object(expr.value)}"
       % endif
       style="width:65%;">
</%def>


<%def name="render_search_updated(expr, desc)">
<%
    key = expr.form_key()
    operators = [E.EQ, E.LT, E.LE, E.GT, E.GE, E.IS_NULL]
    current_op = expr.op
    value = h.format_object(expr.value)
%>
<select name="${key}_type" style="width:30%;">
  % for op in operators:
  ${render_match_operator(op, op == current_op, desc.type)}
  % endfor
</select>
<input type="text" name="${key}_value"
       % if desc.error is not None:
       value="${desc.error.value}"
       class="error_box error mpuls-datepicker"
       % else:
       value="${value}"
       class="mpuls-datepicker"
       % endif
       style="width:65%;">
</%def>


<%def name="render_search_responsibilities(expr, desc)">
<%
    selected = []
    if expr.own:
        selected.append(1)
    if expr.standin:
        selected.append(2)
    if expr.changed_editor:
        selected.append(3)
%>
% if not h.hasRole(['cm']):
<p>
  ${_("Note: This criterion matches cases for which the searching user is editor or standin and thus is only useful for case-management users.")}
</p>
% endif
${render_search_choice(expr, selected, desc)}
</%def>


<%def name="render_search_needle(expr, desc)">
<% key = expr.form_key() %>
<input type="text" name="${key}_value" value="${expr.pattern}"
       style="width:100%;">
</%def>


<%def name="render_search_input(expr, desc)">
% if isinstance(expr, (S.Choice, S.Status)):
  ${render_search_choice(expr, expr.values, desc)}
% elif isinstance(expr, S.Radio):
  ${render_search_radio(expr, expr.value, desc)}
% elif isinstance(expr, S.Bool):
  ${render_search_bool(expr, expr.value, desc)}
% elif isinstance(expr, (S.Field, S.Between)):
  ${render_search_entry(expr, desc)}
% elif isinstance(expr, S.Responsibilities):
  ${render_search_responsibilities(expr, desc)}
% elif isinstance(expr, S.Needle):
  ${render_search_needle(expr, desc)}
% elif isinstance(expr, S.Updated):
  ${render_search_updated(expr, desc)}
% endif
</%def>

<%def name="search_expr(expr, desc)">
<% key = expr.form_key() %>
<div class="panel panel-default">
  <div class="panel-heading" role="tab" id="heading${key}">
    <div class="pull-right">
      <input type="submit" value="${_('Delete')}" name="delete_field_${key}"
             class="link_button">
    </div>
    <h4> <!-- class="panel-title" -->
      <a data-toggle="collapse" data-parent="#search_params"
         aria-expanded="true" aria-controls="collapse${key}"
         href="#collapse${key}">${desc.description}</a>
    </h4>
  </div>
  <% collapsed = "in" if desc.error is not None else "" %>
  <div id="collapse${key}" class="panel-collapse collapse ${collapsed}"
       role="tabpanel" aria-labelledby="heading${key}">
    <div class="panel-body">
      % if desc.error is not None:
      <div class="error">
        ${desc.error}
      </div>
      % endif

      ${self.render_search_input(expr, desc)}
    </div>
  </div>
</div>
</%def>

% if g.mpuls_config.is_enabled('case-module', 'search'):
<form method="post" name="case_search" accept-charset="UTF-8"
      action="${h.url(controller='case_overview', action='search')}">
  ${ h.csrf_token() }
  <div class="widget container">
    <table>
      <tr>
        <td style="overflow: visible;width:60%;padding-right:1em;">
          <h4>${_('New field')}</h4>
          % if c.add_field_error:
            <font color="#A40000">${c.add_field_error}</font><br/>
          % endif
          ${_('Field name')}
          <input type="text" size="30" name="new_field_description" maxlength="256"
                 class="typeahead">
          <input type="submit" value="${_('Add')}" name="add_field"
                 class="button">
          <a class="help" href="${h.url(controller='help', action='help_flexible_search')}"
                 target="_blank">
                 <img width="12" height="12" alt="help" src="${h.url('/images/icons/formular/help.png')}"/>
          </a>
        </td>
      </tr>
      <tr>
        <td style="width:60%;padding-right:1em;">
          <h4>${_('Your current search parameters')}</h4>
          <div class="panel-group" id="search_params" role="tablist"
               aria-multiselectable="true">
            % for expr, extra in c.search_exprs:
            ${search_expr(expr, extra)}
            % endfor
          </div>
        </td>
        <td style="overflow: visible;vertical-align: top;">
          <h4 style="margin-top: 1em;">${_('Saved searches')}</h4>
          <ul>
            % for num, f in enumerate(c.saved_searches):
            <li><a href="${h.url(controller='savedsearch', action='load', id=f.id)}"
                   >${ F.shorten(f.get_name(),60) }</a></li>
            % endfor
          </ul>
          <p><a href="${h.url(controller='savedsearch', action='overview')}"
                >${_('Manage saved searches')}</a></p>
        </td>
      </tr>
    </table>
    <input type="submit" value="${_('Perform search')}" name="perform_search"
           class="button">
    <input type="submit" value="${_('Save current search')}" name="to_json"
           class="button">
    <a href="${h.url(controller='case_overview', action='reset')}"
       class="button">${_('Reset')}</a>
  </div>
</form>
% endif

<%def name="showCount(num)">
  ${"(%s)" % num}
</%def>
<%def name="buildFormHeader()">
  <form method="post" name="case_search" action="${h.url(controller='case_overview', action='search')}" accept-charset="UTF-8">
      ${ h.csrf_token() }
</%def>
<%def name="buildSearchReset()">
  <a href="${h.url('#')}">${_('Reset search')}</a>
</%def>
<%def name="buildSearchExtend()">
  % if c.extended_search:
    <a href="${h.url('#')}">${_('Hide extended search')}</a>
  % else:
    <a href="${h.url('#')}">${_('Show extended search')}</a>
  % endif
  ${h.getHelp('extended_search')}
</%def>
