#!/bin/bash
# -------------------------------------------------------------------
# Copyright (C) 2015 by Intevation GmbH
# Author(s):
# Sascha Wilde <wilde@intevation.de>

# This program is free software under the GNU GPL (>=v2)
# Read the file COPYING coming with the software for details.
# -------------------------------------------------------------------

# Siehe auch:
# mpuls/issue3877 (mpuls_S: Prozessaufbau Nutzermonitoring und Heranführung)

if [ $# -ne 3 ] ; then
    cat <<EOF
Usage:
   `basename $0` FKZ_LIST DATE_RANGE_BEGIN DATE_RANGE_END

Where
   FKZ_LIST is a file with a list of all FKZ to analyse 
   (as generated by dblist.sh)

   DATE_RANGE_BEGIN (inclusive) is the start and
   DATE_RANGE_END (exclusive) is the end date 
   of the time range to analyse. 
   So the range taken into account is
   DATE_RANGE_BEGIN <= date < DATE_RANGE_END 
EOF
    exit -1
fi

FKZ_LIST="$1"
DATE_RANGE_BEGIN="$2"
DATE_RANGE_END="$3"   # exclusive!

# Phases Table:
#    id |               value               
#   ----+-----------------------------------
#    -3 | Unbekannt
#  N -1 | Keine Angabe
#  U 10 | Ungepr?ft
#  E 11 | Entscheidung
#  S 12 | Stipendium
#  V 13 | Weiterbewilligung (unvollst?ndig)
#  V 14 | Weiterbewilligungspr?fung
#  A 15 | Stipendium abgelehnt
#  P 16 | Stipendium pause
#  B 17 | Stipendium beendet

# Status Bearbeitbar: status == 1 in ka_status_tbl
# und Aktualisierung: zugriff in ka_status_tbl im gegebenen Zeitraum.
#
SQL_GET_BASE_SET="SELECT m.phase, s.zugriff FROM master_tbl m, ka_status_tbl s
 WHERE s.master_id = m.id AND s.status = 1 AND
 s.zugriff BETWEEN '$DATE_RANGE_BEGIN' AND '$DATE_RANGE_END'"

SQL_GET_PHASES_SUMS="WITH base AS ($SQL_GET_BASE_SET) SELECT
 (SELECT wert FROM ka_konfiguration_tbl WHERE bez = 'ka_name') AS name,
 (SELECT count(*) FROM base WHERE phase = -1) AS N,
 (SELECT count(*) FROM base WHERE phase = 10) AS U,
 (SELECT count(*) FROM base WHERE phase = 11) AS E,
 (SELECT count(*) FROM base WHERE phase = 12) AS S,
 (SELECT count(*) FROM base WHERE phase = 13 or phase = 14) AS V,
 (SELECT count(*) FROM base WHERE phase = 15) AS A,
 (SELECT count(*) FROM base WHERE phase = 16) AS P,
 (SELECT count(*) FROM base WHERE phase = 17) AS B;"

echo 'FKZ	Name	N	U	E	S	V	A	P	B'
for fkz in $(< "$FKZ_LIST") ; do
  echo -n "${fkz}	"
  psql ka_${fkz}_db -t -A -F '	' -c "$SQL_GET_PHASES_SUMS"
done


# Mittel-Geber:

#    id |         value         
#   ----+-----------------------
#  N -1 | Keine Angabe
#  A  0 | Aquise
#  A  1 | Aquise vollst?ndig
#  F  2 | F?rderung
#  F  3 | F?rderung vollst?ndig
#  S  4 | Stop
#  S  5 | Stop vollst?ndig

SQL_GET_MG_PHASES_SUMS="WITH base AS ($SQL_GET_BASE_SET) SELECT
 (SELECT wert FROM ka_konfiguration_tbl WHERE bez = 'ka_name') AS name,
 (SELECT count(*) FROM base WHERE phase = -1) AS N,
 (SELECT count(*) FROM base WHERE phase = 0 or phase = 1) AS A,
 (SELECT count(*) FROM base WHERE phase = 2 or phase = 3) AS F,
 (SELECT count(*) FROM base WHERE phase = 4 or phase = 5) AS S;"

echo -e '\nMittelgeber:\n'
echo 'FKZ	Name	N	A	F	S'
for fkz in $(< "$FKZ_LIST") ; do
  echo -n "${fkz}	"
  psql ka_mg${fkz}_db -t -A -F '	' -c "$SQL_GET_MG_PHASES_SUMS"
done
