BEGIN;

-- Will transform the current db to a evalserverdb

-- Disable user triggers on DB. Not usefull here because of triggers creating
-- logbook entries
ALTER TABLE master_tbl DISABLE TRIGGER USER;

SET ROLE :adm_ka_owner;

INSERT INTO db_history (beschreibung, zeitpunkt)
       VALUES ('evalserver.sql', current_timestamp);

-- Datasets in the database will not have a specific editor
ALTER table master_tbl DROP CONSTRAINT has_bearbeiter_id_fk;
drop function master_ds_created_logbook() cascade;
drop function master_ds_editor_changed() cascade;
drop function mpuls_log() cascade;
drop function master_lastaccess_log() cascade;
drop function status_lastaccess_log() cascade;
drop function rg_lastaccess_log() cascade;

CREATE OR REPLACE FUNCTION set_lastaccess_time( masterid integer ) 
RETURNS void AS 
$$
BEGIN
END;
$$ LANGUAGE plpgsql VOLATILE;

CREATE TABLE bundeslaender_tbl (
    id INTEGER PRIMARY KEY NOT NULL,
    name VARCHAR(256) NOT NULL UNIQUE,
    kurz CHAR(2) NOT NULL UNIQUE
);

INSERT INTO bundeslaender_tbl (id, name, kurz) VALUES (-1, 'keine Angabe', '');
INSERT INTO bundeslaender_tbl (id, name, kurz) VALUES ( 0, 'Baden-Württemberg', 'BW');
INSERT INTO bundeslaender_tbl (id, name, kurz) VALUES ( 1, 'Bayern', 'BY');
INSERT INTO bundeslaender_tbl (id, name, kurz) VALUES ( 2, 'Berlin', 'BE');
INSERT INTO bundeslaender_tbl (id, name, kurz) VALUES ( 3, 'Brandenburg', 'BB');
INSERT INTO bundeslaender_tbl (id, name, kurz) VALUES ( 4, 'Bremen', 'HB');
INSERT INTO bundeslaender_tbl (id, name, kurz) VALUES ( 5, 'Hamburg', 'HH');
INSERT INTO bundeslaender_tbl (id, name, kurz) VALUES ( 6, 'Hessen', 'HE');
INSERT INTO bundeslaender_tbl (id, name, kurz) VALUES ( 7, 'Mecklenburg-Vorpommern', 'MV');
INSERT INTO bundeslaender_tbl (id, name, kurz) VALUES ( 8, 'Niedersachsen', 'NI');
INSERT INTO bundeslaender_tbl (id, name, kurz) VALUES ( 9, 'Nordrhein-Westfalen', 'NW');
INSERT INTO bundeslaender_tbl (id, name, kurz) VALUES (10, 'Rheinland-Pfalz', 'RP');
INSERT INTO bundeslaender_tbl (id, name, kurz) VALUES (11, 'Saarland', 'SL');
INSERT INTO bundeslaender_tbl (id, name, kurz) VALUES (12, 'Sachsen', 'SN');
INSERT INTO bundeslaender_tbl (id, name, kurz) VALUES (13, 'Sachsen-Anhalt', 'ST');
INSERT INTO bundeslaender_tbl (id, name, kurz) VALUES (14, 'Schleswig-Holstein', 'SH');
INSERT INTO bundeslaender_tbl (id, name, kurz) VALUES (15, 'Thüringen', 'TH');

GRANT SELECT ON bundeslaender_tbl TO GROUP :cm_group;
GRANT UPDATE ON bundeslaender_tbl TO GROUP :cm_group;
GRANT SELECT ON bundeslaender_tbl TO GROUP :admin_group;

CREATE TABLE fkz_tbl (
    id            INTEGER PRIMARY KEY NOT NULL,
    name          VARCHAR(256) NOT NULL,
    fkz           VARCHAR(256) NOT NULL UNIQUE,
    database      VARCHAR(256) NOT NULL UNIQUE,
    bundesland_id INTEGER NOT NULL REFERENCES bundeslaender_tbl(id),
    job_percentage REAL DEFAULT NULL,
    UNIQUE (fkz, database)
);

GRANT SELECT ON fkz_tbl TO GROUP :cm_group;
GRANT UPDATE ON fkz_tbl TO GROUP :cm_group;
GRANT SELECT ON fkz_tbl TO GROUP :admin_group;

CREATE INDEX master_id_idx on ka_status_tbl (master_id);

-- Tabelle für die Zuordnung von Benutzern zu Einrichtungen (Multiuser
-- Auswertungsserver). Diese kann verwendet werden, um zu beschränken,
-- welcher Nutzer welche Agenturen sehen darf. Dafür ist es allerdings
-- nötig, fkz_tbl_view entsprechend zu definieren, was in der
-- default-Definition nicht der Fall ist, da es nicht häufig benötigt
-- wird.
-- FIXME: Es wäre wohl sinnvoll, nm_user_fkz ganz zu entfernen, aber das
-- verlangt auch Änderungen in base.
CREATE TABLE nm_user_fkz (
    id      SERIAL PRIMARY KEY NOT NULL,
    user_id INTEGER NOT NULL REFERENCES ka_benutzer_tbl(id) ON DELETE CASCADE,
    fkz_id  INTEGER NOT NULL REFERENCES fkz_tbl(id) ON DELETE CASCADE
);

GRANT SELECT ON nm_user_fkz TO GROUP :cm_group;
GRANT USAGE ON nm_user_fkz_id_seq TO GROUP :admin_group;
GRANT INSERT ON nm_user_fkz TO GROUP :admin_group;
GRANT DELETE ON nm_user_fkz TO GROUP :admin_group;
GRANT SELECT ON nm_user_fkz TO GROUP :admin_group;

-- Berechtigungen auch für das direkte Eintragen von Vertretungsregelungen, so
-- das nicht mehr die create_ds und delete_ds Funktionen genutzt werden
-- müssen.
GRANT INSERT ON nm_benutzer_master_tbl TO GROUP :admin_group;
GRANT DELETE ON nm_benutzer_master_tbl TO GROUP :admin_group;
GRANT USAGE ON nm_benutzer_master_tbl_id_seq TO GROUP :admin_group;

-- Nur Einrichtungen anzeigen für die der Nutzer auf die Entsprechenden Rechte
-- hat.
CREATE OR REPLACE VIEW fkz_tbl_view
   AS SELECT DISTINCT f.* from fkz_tbl f;

GRANT SELECT ON fkz_tbl_view TO GROUP :cm_group;
GRANT SELECT ON fkz_tbl_view TO GROUP :admin_group;

-- Default evaluation view für den Auswertungsserver, ohne Überprüfung ob der
-- Nutzer wirklich auch Bearbeiter ist
DROP VIEW master_tbl_eval_total_view;
CREATE OR REPLACE VIEW master_tbl_eval_total_view
   AS SELECT m.*
   FROM master_tbl_view m JOIN ka_status_tbl s
   ON m.id = s.master_id
   WHERE s.status <> 3;
GRANT SELECT ON master_tbl_eval_total_view TO GROUP :cm_group;
GRANT SELECT ON master_tbl_eval_total_view TO GROUP :admin_group;

SET ROLE postgres;
ALTER TABLE master_tbl ENABLE TRIGGER USER;
COMMIT;
