#! /usr/bin/python
# -*- coding: utf-8 -*-
import json
import sys
import os
import traceback
import decimal
from optparse import OptionParser
from datetime import date, datetime, timedelta
import calendar

sys.path.append('/home/mpuls/jmd-auswertung/releases/current')
sys.path.append('/home/mpuls/jmd-auswertung/releases/current/libmpuls')
sys.path.append('/home/mpuls/jmd-auswertung/releases/current/base')
sys.path.append('/home/mpuls/jmd-auswertung/releases/current/libformed')

# For get_xml data
from libmpuls.evaluation.config import EvaluationConfig
from libmpuls.evaluation.evaluation import EvaluationSet
from libmpuls.evaluation.export import EvaluationExportXML
from libmpuls.db.db import get_db

# For parsing XML
import xml.etree.ElementTree as ET

# For writing xls
import xlwt
date_style = xlwt.XFStyle()
date_style.num_format_str = "DD.MM.YYYY"

# For sending the emails.
import smtplib
from email import encoders
from email.mime.multipart import MIMEMultipart
from email.mime.base import MIMEBase
from email.mime.text import MIMEText

# For cleaning files
import glob

###
# 1. Get fkzs for a carrier
# 2. get xmls for a fkz
# 3. Parse xmls
# 4. Create XSL
# 5. Send mail
###

### Constants to configure
WORKDIR="/tmp/auswertungen-fallakten"

def load_config(config_file_name):
    with open(config_file_name) as config_file:
        config = json.load(config_file)
    map(lambda x:set_global_variable(x[0], x[1]), config.iteritems())


def set_global_variable(key, value):
    globals()[key] = value



### Read the fkzs for a carrier from the db

def get_fkzs(db, case_nr):
    # Takes a carrier and collects the fkzs for this carrier
    sql_template = """SELECT fkz, database, name
                        FROM fkz_tbl
                       WHERE {condition} AND name NOT LIKE '%%(bis%%'
                    ORDER BY fkz;"""
    if case_nr is not None:
        condition = "bundesland_id >= 0 AND fkz::integer/1000 = %s"
        params = (case_nr,)
    else:
        condition = "bundesland_id < 0"
        params = ()
    sql = sql_template.format(condition=condition)

    try:
        conn = db.getConnection()
        cur = conn.cursor()
        try:
            cur.execute(sql, params)
            rows = [(row[0], row[1], row[2]) for row in cur]
            # Restrict here for testing reasons e.g. rows[:3]
            return rows
        finally:
            db.recycleConnection(conn, cur)
    except Exception, err:
        #log.error('Error on evaluation set: %s' % err)
        traceback.print_exc()
        print "Failed to get connection. Check credetials"
    return []


### Gets the xmls and write them down.

def write_eval_to_tmp(db, start_date, end_date):
    """Perform evaluations and write the results to XML files.
    The return value is a mapping from carrier (the keys in
    carrier_mapper) to the list of the generated XML files for that carrier.
    """
    xml_files = {}
    for carrier in carrier_mapper:
        print "* Processing", carrier
        case_nr = carrier_mapper[carrier]['case_nr']
        carrier_files = []
        fkzs = get_fkzs(db, case_nr)
        for (fkz, dbname, name) in fkzs:
            print "** fkz ", fkz
            for (ev, ev_path) in eval_mapper:
                print "*** evaluation", ev
                xml = get_xml(db, fkz, ev_path, start_date, end_date,
                              case_nr is None)
                filename = '%s_%s_%s.xml' % (carrier, fkz, ev)
                with open(filename, 'wb') as f1:
                    f1.write(xml)
                carrier_files.append((fkz, ev, filename))
        xml_files[carrier] = carrier_files
    return xml_files


def get_xml(db, fkz, evaluation_path, start_date, end_date, group_fkzs):
    sql_template = """\
        SELECT %(fields)s
          FROM master_tbl_eval_total_view m
         WHERE m.id IN (SELECT x.id FROM master_tbl_eval_total_view x
                          JOIN fkz_tbl f
                            ON {condition}
                         WHERE f.fkz='{fkz}')
    """
    if group_fkzs:
        condition = "((x.fkz :: INTEGER) / 1000) = f.id"
    else:
        condition = "x.fkz::VARCHAR = f.id::VARCHAR"
    sql = sql_template.format(condition=condition, fkz=fkz)

    # get the XML of an evaluation
    conn, cur = None, None
    try:
        try:
            conn = db.getConnection()
            config = EvaluationConfig(
                evaluation_path,
                conn,
                start_date.isoformat(),
                end_date.isoformat(),
                eval_config["sdatefield"],
                eval_config["edatefield"],
                eval_config["pphase"],  # Start-phase
                sql,
                None,
                fkz,
                None,
                adele=True)
            evaluationset = EvaluationSet(config, False)
            evaluationset.evaluate()
            result = evaluationset.export(EvaluationExportXML())
            result_encoded = result.encode('utf-8')
            return result_encoded
        except Exception, err:
            traceback.print_exc()
            print "Failed to get connection. Check credetials"
    finally:
        db.recycleConnection(conn, cur)
        db.closeConnections()


### Code for parsing the xml

class EvaluationResult(object):
    def __init__(self, name, id1, fkz):
        self.name = name
        self.id1 = id1
        self.h1 = ['']
        self.h2 = ['']
        self.h3 = ['FKZ']
        self.h4 = ['']
        self.line = [fkz]

class ParsedEvaluation(object):
    def __init__(self, name):
        self.name = name
        self.lines = []

def parse_xml(xml_files, ev, short_names):
    # parse the xmls for one evaluationset
    start = True # at the beginning we want a list of ParsedEvaluations
    results = []
    for (fkz, file_ev, xml_file) in xml_files:
        if file_ev != ev:
            continue
        for idx, input1 in enumerate(parse_xml_of_one_file(fkz, ev, xml_file)):
            if start:
                if short_names:
                    name = input1.name
                else:
                    name = "(%s)_%s" % (input1.id1, ev)
                parsed_evaluation = ParsedEvaluation(name)
                parsed_evaluation.lines.append(input1.h1)
                parsed_evaluation.lines.append(input1.h2)
                parsed_evaluation.lines.append(input1.h3)
                parsed_evaluation.lines.append(input1.h4)
                results.append(parsed_evaluation)
            results[idx].lines.append(input1.line)
        start = False
    return results


def parse_xml_of_one_file(fkz, ev, xml_file):
    tree = ET.parse(xml_file)
    doc = tree.getroot()
    results = []
    for evaluation in doc.findall('evaluation'):
        result = EvaluationResult(evaluation.attrib['short_name'],
                                  evaluation.attrib['id'], fkz)
        result.h1.append(evaluation.attrib['name'])
        for table in evaluation.findall('table'):
            result.h2.append(table.attrib['ylabel'])
            headers = [header.text for header in table.findall('headers/header')]
            for row in table.findall('rows/row'):
                result.h3.append(row.attrib['label'])
                for index, col in enumerate(row.findall('col')):
                    result.h4.append(headers[index % len(headers)])
                    result.line.append(col.text)

                # update header 3
                while len(result.h3) < len(result.h4):
                    result.h3.append('')

                # update header1
                while len(result.h1) < len(result.h4):
                    result.h1.append('')

                # update header2
                while len(result.h2) < len(result.h4):
                    result.h2.append('')

        results.append(result)
    return results


### Code for writing the parsed data into the xls

def write_to_xls(xml_files, short_names, start_date, end_date):
    filenames = {}
    print "* Generating xls"
    for carrier in carrier_mapper:
        print "** ", carrier
        wb = xlwt.Workbook()
        ws = wb.add_sheet('Metadaten')
        ws.write(0,0, EMAIL_SUBJECT)
        ws.write(1,0, 'Datum')
        ws.write(1,1, date.today(), date_style)
        ws.write(2,0, 'Start')
        ws.write(2,1, start_date, date_style)
        ws.write(3,0, 'Ende')
        ws.write(3,1, end_date, date_style)
        for (ev, ev_path) in eval_mapper:
            for pe in parse_xml(xml_files[carrier], ev, short_names):
                ws = wb.add_sheet(pe.name)
                if len(pe.lines[0]) > 255:
                    cutting = []
                    lines = pe.lines
                    while len(lines[0]) > 255:
                        cut1 = [line[0:255] for line in lines]
                        lines = [line[255:] for line in lines]
                        cutting.append(cut1)
                    if len(lines[0]) > 0:
                        cutting.append(lines)
                    for k, cut in enumerate(cutting):
                        sheet_name = pe.name + "_" + str(k + 1)
                        ws = wb.add_sheet(sheet_name)
                        write_to_sheet(ws, cut)
                else:
                    write_to_sheet(ws, pe.lines)
        filename = FILENAME_PATTERN % (carrier,
                                       start_date.isoformat(),
                                       end_date.isoformat(),
                                       date.today().isoformat())
        wb.save(filename)
        filenames[carrier] = filename
    return filenames


def write_to_sheet(ws, lines):
    for (rownr, row) in enumerate(lines):
        for (colnr, col) in enumerate(row):
            try:
                out = decimal.Decimal(col)
            except decimal.InvalidOperation:
                out = col
            ws.write(rownr, colnr, out)


### Code for sending the mails

text_template = u"""\
Anbei erhalten Sie die %(title)s
für den Zeitraum %(start_date)s bis zum %(end_date)s als xls-Datei.

Viele Grüße
Ihr mpuls-Team
"""

def send_mail(carrier, emails, xls_filename, options):
    print "* send_mails", carrier
    outer = MIMEMultipart()
    outer['Subject'] = EMAIL_SUBJECT
    outer['To'] = ', '.join(emails)
    outer['From'] = 'mpuls@intevation.de'

    outer.attach(MIMEText(text_template
                          % {"title": EMAIL_SUBJECT,
                             "start_date":
                             options.period.start.strftime("%d.%m.%Y"),
                             "end_date":
                             options.period.end.strftime("%d.%m.%Y")},
                          _charset="latin1"))

    with open(xls_filename) as fp:
        msg = MIMEBase('application', 'octet-stream')
        msg.set_payload(fp.read())
    encoders.encode_base64(msg)

    msg.add_header('Content-Disposition', 'attachment', filename=xls_filename)
    outer.attach(msg)

    # Delete the files
    if not options.keep_files:
        os.remove(xls_filename)
        for f1 in glob.glob('%s*.xml' % carrier):
            os.remove(f1)

    composed = outer.as_string()
    # Write to disk or send per email
    if options.send_email:
        s = smtplib.SMTP('localhost')
        recipients = emails
        if options.forced_recipient is not None:
             recipients = [options.forced_recipient]
        s.sendmail('mpuls@intevation.de', recipients, composed)
        s.quit()
    else:
        with open('tmp-email-%s.txt' % carrier, 'w') as fp:
            fp.write(composed)


class Period(object):

    """A period of time given by start and end dates"""

    def __init__(self, start, end):
        self.start = start
        self.end = end

    def __repr__(self):
        return "Period(%r, %r)" % (self.start, self.end)

    @classmethod
    def year(cls, year=None):
        """Return a Period instance covering the given year.
        If not year is given, the year defaults to the current year.
        """
        if year is None:
            year = date.today().year
        return cls(date(year=year, month=1, day=1),
                   date(year=year, month=12, day=31))

    @classmethod
    def quarter_of_date(cls, d):
        """Return a period covering the quarter containing date."""
        start = d.replace(month=((d.month - 1) // 3) * 3 + 1, day=1)
        end_month = start.month + 2
        end = d.replace(month=end_month,
                        day=calendar.monthrange(start.year, end_month)[1])
        return cls(start, end)

    @classmethod
    def current_quarter(cls):
        """Return a period covering the current quarter."""
        return cls.quarter_of_date(date.today())

    @classmethod
    def previous_quarter(cls):
        """Return a period covering the previous quarter."""
        current_quarter = cls.current_quarter()
        return cls.quarter_of_date(current_quarter.start - timedelta(1))

    @classmethod
    def from_unit(cls, unit, year, previous_quarter):
        """Return a period based on unit, year and previous_quarter.

        If unit is 'year', return the year indicated by year.

        If unit is 'quarter', return the current quarter if
        previous_quarter is false, else the previous quarter.

        If unit is 'year-quarter', return a period starting with the
        start of the the given year and ending at the current or
        previous quarter, depending on previous_quarter.
        """
        if unit == "year":
            return cls.year(year)

        if previous_quarter:
            quarter = cls.previous_quarter()
        else:
            quarter = cls.current_quarter()

        if unit == "quarter":
            return quarter
        elif unit == "year-quarter":
            quarter.start = date(year, 1, 1)
            return quarter
        else:
            raise ValueError("Unknown unit %r" % (unit,))


# Main function

def main():
    #
    # Parse user options
    #
    parser = OptionParser()
    parser.set_defaults(period=None,
                        duration_unit="year",
                        previous_quarter=False,
                        year=date.today().year,
                        short_names=False)
    parser.add_option("--duration-unit",
                      action="store",
                      choices=("year", "quarter", "year-quarter"),
                      help=("The unit in which the duration is specified."
                            " 'year' (the default) means an entire year,"
                            " 'quarter' means a single quarter and"
                            " 'year-quarter' means start of a year until end"
                            " of a quarter."
                            " The precise start/end dates are influenced by"
                            " --year and --previous-quarter."))
    parser.add_option("--year",
                      action="store",
                      type="int",
                      help=("The year to use for the duration units 'year'"
                            " and 'year-quarter'. Default: current year"))
    parser.add_option("--previous-quarter",
                      action="store_true",
                      help=("Use previous instead of current quarter for"
                            " duration units 'quarter' and 'year-quarter'."))
    parser.add_option("--short-names",
                      action="store_true",
                      help=("Use the short names of the evaluations as titles"
                            " of the worksheets in the XLS files."))
    parser.add_option("--dont-send-email",
                      dest="send_email",
                      action="store_false",
                      default=True,
                      help=("Don't send emails with the reports but write them"
                            " to a file instead 'tmp-email-$TRAEGER.txt'"))
    parser.add_option("--keep-files",
                      action="store_true",
                      dest="keep_files",
                      default=False,
                      help="Don't delete generated files")
    parser.add_option("--force-recipient",
                      action="store",
                      dest="forced_recipient",
                      default=None,
                      help=("Send mails only to this address for"
                            " for easier testing"))
    parser.add_option("--config",
                      action="store",
                      dest="config_file",
                      default=None,
                      help="Configuraton file - mandatory")

    (options, args) = parser.parse_args()

    options.period = Period.from_unit(options.duration_unit, options.year,
                                      options.previous_quarter)


    if not options.config_file:
        print("Configuration file missing. Aborting.")
        exit(2)
    else:
        load_config(options.config_file)

    if not os.access(WORKDIR, os.F_OK):
        os.mkdir(WORKDIR)
    os.chdir(WORKDIR)

    print "\n** Start %s" % datetime.now()
    # Create db with user infos
    db = get_db(USERNAME, PASSWORD, DBNAME, HOST, PORT)

    # Main stuff
    xml_files = write_eval_to_tmp(db, options.period.start, options.period.end)
    xls_filenames = write_to_xls(xml_files, options.short_names,
                                 options.period.start, options.period.end)
    for carrier in carrier_mapper:
        send_mail(carrier, carrier_mapper[carrier]['emails'],
                  xls_filenames[carrier], options)

    print "** End %s" % datetime.now()

# Test code

if __name__=='__main__':
    main()
