# -*- coding: utf-8 -*-
#
# Copyright (c) 2007, 2013 by Intevation GmbH
#
# FormEd is free software under the terms of the
# GNU General Public License Version 3. See file
# LICENSE coming with the source of FormEd for details.
#
# Authors:
# Sascha L. Teichmann <teichmann@intevation.de>
# Björn Ricks <bjoern.ricks@intevation.de>
#


import formed.meta.data as data
from formed.meta.nodecomponents import Node, Leaf

from string import Template

import codecs

SQL_TEMPLATE = Template('''
--
-- THIS IS MACHINE GENERATED BY libformed. BE CAREFUL WITH MANUAL EDITING
--
-- Version: $$Revision: 0fdbb639be0a $$
--
BEGIN;

-- ------ This is static code and therefore may be put into a separate file. -------

-- Used to track the structure changes of a case
-- DROP TABLE case_structure CASCADE;

CREATE TABLE case_structure (
    id        SERIAL PRIMARY KEY NOT NULL,
    master_id int4 NOT NULL REFERENCES master_tbl(id) ON DELETE CASCADE,
    modified  boolean,
    cache     TEXT
);

-- each time a new case is created create a new entry in the strcuture table

-- DROP TRIGGER case_insert_trigger ON master_tbl;
-- DROP FUNCTION case_insert_func();

CREATE OR REPLACE FUNCTION case_insert_func() RETURNS TRIGGER AS $$$$
BEGIN
    INSERT INTO case_structure (master_id, modified) VALUES (NEW.id, true);
    RETURN NEW;
END;
$$$$ LANGUAGE plpgsql;

CREATE TRIGGER case_insert_trigger AFTER INSERT ON master_tbl
FOR EACH ROW EXECUTE PROCEDURE case_insert_func();

-- ------ end of static code -------

-- PLPython is an untrusted language. -> Need to be postgres.
SET ROLE postgres;

-- DROP FUNCTION compute_case_structure(int4);
CREATE OR REPLACE FUNCTION compute_case_structure(case_id int4) RETURNS TEXT
AS $$$$

class Node(object):

    def __init__(self, name, id=None, children=[]):
        self.name = name
        self.id = id
        self.children = children

    def to_json(self, out):
        out.append('{"name":"%s","id":%d' % (self.name, self.id))
        if self.children:
            out.append(',"children":[')
            first = True
            for child in self.children:
                if first:
                    first = False
                else:
                    out.append(",")
                child.to_json(out)
            out.append("]")
        out.append("}")

    def recursive_build(self, mid, children, parent_name="master"):
        select = ("SELECT id FROM %s WHERE %s_id = %d"
                  % (self.name, parent_name, mid))
        ids = [r['id'] for r in plpy.execute(select)]
        for id in ids:
            sub_children = []
            node = Node(self.name, id, sub_children)
            name = self.name[:-4] # cut off trailing '_tbl'
            for child in self.children:
                child.recursive_build(id, sub_children, name)
            children.append(node)

TREE_STRUCTURE = \
$TREE_STRUCTURE


children = []
tree = Node(TREE_STRUCTURE.name, case_id, children)
for child in TREE_STRUCTURE.children:
    child.recursive_build(case_id, children)
out = []
tree.to_json(out)
return "".join(out)

$$$$ LANGUAGE plpythonu EXTERNAL SECURITY DEFINER;
ALTER FUNCTION compute_case_structure(int4) OWNER TO :adm_ka_owner;

-- DROP FUNCTION get_case_structure(int4);
CREATE OR REPLACE FUNCTION get_case_structure(case_id int4) RETURNS TEXT AS $$$$

r = plpy.execute("SELECT modified, cache FROM case_structure "
                 "WHERE master_id = %d" % case_id, 1)

if not r:
    return ""
r = r[0]

if r['modified']: # need refresh
    structure = plpy.execute("SELECT compute_case_structure(%d) as s;"
                             % case_id, 1)[0]["s"]
    plan = plpy.prepare("UPDATE case_structure SET modified = false, "
                        "cache = $$1 WHERE master_id = $$2", ["text", "int4"])
    plpy.execute(plan, [structure, case_id])
    return structure

return r['cache']

$$$$ LANGUAGE plpythonu EXTERNAL SECURITY DEFINER;
ALTER FUNCTION get_case_structure(int4) OWNER TO :adm_ka_owner;

-- Continue as db-owner from here
SET ROLE :adm_ka_owner;

--
-- These triggers are used to keep case_structure table in sync.
-- Each time a repeat group is created or deleted the case_structure
-- modified field has to be set to 'true'.
--

$TRIGGERS

$CREATE_DELETE_FUNCTIONS

COMMIT;
''')

TRIGGER_TMPL = Template('''
-- DROP TRIGGER ${RELATION}_insert_trigger ON ${RELATION} CASCADE;
-- DROP FUNCTION ${RELATION}_insert_func();
-- DROP TRIGGER ${RELATION}_delete_trigger ON ${RELATION} CASCADE;
-- DROP FUNCTION ${RELATION}_delete_func();

CREATE OR REPLACE FUNCTION ${RELATION}_insert_func() RETURNS TRIGGER AS $$$$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid ${RG_MASTER_INSERT};
    UPDATE case_structure
        SET modified = FALSE, cache = compute_case_structure(master_id)
        WHERE master_id = mid;
    PERFORM set_lastaccess_time(mid);
    RETURN NEW;
END;
$$$$ LANGUAGE plpgsql;

CREATE TRIGGER ${RELATION}_insert_trigger AFTER INSERT ON ${RELATION}
    FOR EACH ROW EXECUTE PROCEDURE ${RELATION}_insert_func();

CREATE OR REPLACE FUNCTION ${RELATION}_update_func() RETURNS TRIGGER AS $$$$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid ${RG_MASTER_INSERT};
    PERFORM set_lastaccess_time(mid);
    RETURN NEW;
END;
$$$$ LANGUAGE plpgsql EXTERNAL SECURITY DEFINER;

CREATE TRIGGER ${RELATION}_update_trigger AFTER UPDATE ON ${RELATION}
    FOR EACH ROW EXECUTE PROCEDURE ${RELATION}_update_func();

CREATE OR REPLACE FUNCTION ${RELATION}_delete_func() RETURNS TRIGGER AS $$$$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid ${RG_MASTER_DELETE};
    UPDATE case_structure
        SET modified = FALSE, cache = compute_case_structure(master_id)
        WHERE master_id = mid;
    PERFORM set_lastaccess_time(mid);
    RETURN OLD;
END;
$$$$ LANGUAGE plpgsql;

CREATE TRIGGER ${RELATION}_delete_trigger AFTER DELETE ON ${RELATION}
    FOR EACH ROW EXECUTE PROCEDURE ${RELATION}_delete_func();
''')

RG_MASTER_ID_TMPL = Template('''\
master_tbl.id FROM $RELATION
    $INNER_JOINS
    WHERE $RELATION.id = $NEW_OLD.${PARBASE}_id
''')

CREATE_DELETE_TMPL = Template('''
CREATE OR REPLACE FUNCTION create_${RELATION}(pid integer, nuuid char(36))
RETURNS integer
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$$$
DECLARE
    rid integer;
    mid integer;
BEGIN
    ${SELECT_MASTER_ID}

    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    INSERT INTO ${RELATION} (${PARBASE}_id) VALUES (pid);

    SELECT INTO rid currval('${RELATION}_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE ${RELATION}
        SET    uuid_id = nuuid
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('${RELATION}', rid);
        ${CREATE_CHILDREN}
    END IF;

    RETURN rid;
END;
$$$$;

CREATE OR REPLACE FUNCTION delete_${RELATION}(rid integer)
RETURNS void
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$$$
DECLARE
    mid integer;
BEGIN
    ${SELECT_MASTER_ID_DELETE}

    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    DELETE FROM ${RELATION} WHERE id = rid;
END;
$$$$;
''')

CREATE_DELETE_MASTER_TMPL = Template('''
CREATE OR REPLACE FUNCTION create_master_tbl(nuuid char(36))
    RETURNS integer
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$$$
DECLARE
    standin integer;
    rid     integer;
BEGIN
    rid := create_master_ds(nuuid);
    IF nuuid IS NULL THEN
        ${CREATE_CHILDREN}
    END IF;

    RETURN rid;
END;
$$$$;

REVOKE ALL ON FUNCTION create_master_tbl(char(36)) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION create_master_tbl(char(36))
    TO GROUP :cm_group;

CREATE OR REPLACE FUNCTION delete_master_tbl(mid integer) RETURNS void
    LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER
    AS $$$$
DECLARE
    st integer;
BEGIN
    SELECT INTO st s.status FROM master_tbl m JOIN ka_status_tbl s
        ON s.master_id = m.id WHERE m.id = mid;

    IF NOT st in (3, 5) THEN
        RAISE EXCEPTION 'case is not deletable';
    END IF;

    DELETE FROM master_tbl WHERE id = mid;
END;
$$$$;

REVOKE ALL ON FUNCTION delete_master_tbl(integer) FROM PUBLIC;
GRANT EXECUTE ON FUNCTION delete_master_tbl(integer)
    TO GROUP :admin_group;
''')


class RGNode(object):

    def __init__(self, name, min=1, children=None, parent=None):
        self.basename = name
        self.name = "%s_tbl" % name.replace('-', '_')
        self.min = min
        if children is None:
            children = []
        self.children = children
        self.parent = parent

    def to_tree_structure(self, out, indent=0):
        ind = "    " * indent

        if not self.children:
            out.append("%sNode('%s')" % (ind, self.name))
        else:
            out.append("%sNode('%s', children = [\n" % (ind, self.name))
            for idx, child in enumerate(self.children):
                child.to_tree_structure(out, indent + 1)
                out.append((idx < len(self.children) - 1) and ",\n" or "\n")
            out.append("%s])" % ind)

    def create_create_delete(self, out):
        if self.children:
            cout = []
            for child in self.children:
                for i in range(child.min):
                    cout.append("PERFORM create_%s(rid, NULL);" % child.name);
            create_children = "\n      ".join(cout);
        else:
            create_children = ""

        if self.name == "master_tbl":
            out.append(CREATE_DELETE_MASTER_TMPL.substitute({
                'CREATE_CHILDREN': create_children
                }))
        else:
            if self.parent.name == "master_tbl":
                select_master = "mid := pid;"
                select_master_delete = ("SELECT INTO mid master_id"
                                        " FROM %s WHERE id = rid;" % self.name)
            else:
                current = self.parent
                cout = ["SELECT INTO mid master_tbl.id FROM %s" % current.name]
                while current.parent:
                    cout.append("INNER JOIN %(parent)s"
                                " ON %(current)s.%(parbase)s_id = %(parent)s.id"
                                % {'current': current.name,
                                   'parbase': current.parent.basename,
                                   'parent': current.parent.name})
                    current = current.parent

                cout.append("WHERE %s.id = pid;" % self.parent.name)

                select_master = "\n        ".join(cout)

                cout = ["SELECT INTO mid master_tbl.id FROM %s" % self.name]
                current = self
                while current.parent:
                    cout.append("INNER JOIN %(parent)s"
                                " ON %(current)s.%(parbase)s_id = %(parent)s.id"
                                % {'current': current.name,
                                   'parbase': current.parent.basename,
                                   'parent' : current.parent.name})
                    current = current.parent

                cout.append("WHERE %s.id = rid;" % self.name)

                select_master_delete = "\n        ".join(cout)

            out.append(CREATE_DELETE_TMPL.substitute({
                'RELATION': self.name,
                'PARBASE': self.parent.basename,
                'CREATE_CHILDREN': create_children,
                'SELECT_MASTER_ID': select_master,
                'SELECT_MASTER_ID_DELETE': select_master_delete
                }))

        for child in self.children:
            child.create_create_delete(out)


    def create_triggers(self, out):
        if self.parent: # only when not master_tbl
            current = self.parent
            inner_joins = []
            while current.parent:
                inner_joins.append("INNER JOIN %(parent)s ON "
                                   "%(current)s.%(parbase)s_id = %(parent)s.id"
                                   % {'current': current.name,
                                      'parbase': current.parent.basename,
                                      'parent': current.parent.name})
                current = current.parent
            inner_joins = "\n        ".join(inner_joins)

            if inner_joins:
                rg_master_id_insert = RG_MASTER_ID_TMPL.substitute({
                    "RELATION": self.parent.name,
                    "PARBASE": self.parent.basename,
                    "INNER_JOINS": inner_joins,
                    "NEW_OLD": "NEW",
                    })
                rg_master_id_delete = RG_MASTER_ID_TMPL.substitute({
                    "RELATION": self.parent.name,
                    "PARBASE": self.parent.basename,
                    "INNER_JOINS": inner_joins,
                    "NEW_OLD": "OLD",
                    })
            else:
                # shortcut for the common case of repeat groups that are
                # direct children of the master_tbl.  In this case we
                # can compare the master_tbl.id directly with the
                # repeatgroup's master_id
                rg_master_id_insert = "NEW.master_id"
                rg_master_id_delete = "OLD.master_id"

            out.append(TRIGGER_TMPL.substitute({
                "RELATION": self.name,
                "RG_MASTER_INSERT": rg_master_id_insert,
                "RG_MASTER_DELETE": rg_master_id_delete,
                }))
        for child in self.children:
            child.create_triggers(out)


class RGCreator():

    def __init__(self):
        pass

    def createRGs(self, root):
        self.stack = [RGNode("master", 1, [])]
        self._createRecursive(root)
        return self.stack.pop()

    def _createRecursive(self, node):
        for child in node.children:
            self._createChild(child)

    def _createRepeatNode(self, child):
        rg = RGNode(child.getName(), child.getInitial(), [], self.stack[-1])
        self.stack.append(rg)
        self._createRecursive(child)
        self.stack.pop()
        self.stack[-1].children.append(rg)

    def _createChild(self, child):
        if isinstance(child, Leaf):
            return
        elif isinstance(child, Node):
            if isinstance(child, data.RepeatNode):
                self._createRepeatNode(child)
            else:
                self._createRecursive(child)


class ExportAsRepeatGroupSQLFilter:

    def __init__(self, document):
        self.root = document.root

    @staticmethod
    def getDescription():
        return "Export document as repeat group SQL statements"

    def doExport(self, path):
        root = self.root
        creator = RGCreator()
        rg_root = creator.createRGs(root)

        out = []
        rg_root.to_tree_structure(out)
        tree_structure = "".join(out)

        out = []
        rg_root.create_triggers(out)
        triggers = "".join(out)

        out = []
        rg_root.create_create_delete(out)
        create_delete = "".join(out)

        f = codecs.open(path, "w", "UTF-8")
        try:
            print >> f, SQL_TEMPLATE.substitute({
                "TREE_STRUCTURE": tree_structure,
                "TRIGGERS": triggers,
                "CREATE_DELETE_FUNCTIONS": create_delete
                })
        finally:
            f.close()

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8 enc=utf-8:
