<?xml version="1.0" encoding="iso-8859-1"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:output method="xml" omit-xml-declaration="yes" encoding="UTF-8"/>
<xsl:strip-space elements="*"/>

<xsl:key name="xlabels" match="table" use="@xlabel"/>
<xsl:key name="ylabels" match="table" use="@ylabel"/>
<xsl:template match="/">

<xsl:apply-templates />
</xsl:template>

<xsl:template match="evaluationset">
  <div class="evaluation">
    <div class="header">
      <h1>${_('Evaluation Result')}</h1>
    </div>
    <div>
     <xsl:apply-templates />
    </div>
  </div>
</xsl:template>

<xsl:template match="evaluationset/name">
  <strong>${_('Name')}:</strong>
  <xsl:value-of select="." />
  <br/>
</xsl:template>

<xsl:template match="evaluationset/desc">
  <strong>${_('Desc')}:</strong>
  <xsl:value-of select="." />
  <br/>
</xsl:template>

<xsl:template match="evaluationset/start_date">
  <strong>${_('From')}:</strong>
  <xsl:value-of select="." />
  <br/>
</xsl:template>

<xsl:template match="evaluationset/end_date">
  <strong>${_('Until')}:</strong>
  <xsl:value-of select="." />
  <br/>
</xsl:template>

<xsl:template match="evaluation">
  <div class="widget container">
    <h2><xsl:value-of select="./@name" /> (<xsl:value-of select="./@id" />)</h2>
    <xsl:apply-templates />
    <table>
        <xsl:call-template name="headers"/>
        <xsl:call-template name="ydim"/>
    </table>
  </div>
</xsl:template>

<xsl:template match="evaluation/desc">
  <p><xsl:value-of select="."/></p>
</xsl:template>

<xsl:template name="headers">
  <tr>
    <td colspan="2"></td>
    <xsl:call-template name="xlabelgroupings"/> 
  </tr>
  <tr>
    <th colspan="2"></th>
    <xsl:call-template name="xlabels"/> 
  </tr>
</xsl:template>

<xsl:template name="xlabelgroupings">
  <xsl:for-each select="table[generate-id() = generate-id(key('xlabels',@xlabel)[1])]">
    <td colspan="{count(headers/header)}" align="center"><xsl:value-of select="@xlabel"/></td>
  </xsl:for-each>
</xsl:template>

<xsl:template name="xlabels">
  <xsl:for-each select="table[generate-id() = generate-id(key('xlabels',@xlabel)[1])]">
      <xsl:apply-templates select="headers/header"/> 
  </xsl:for-each>
</xsl:template>


<xsl:template match="header">
  <th><xsl:value-of select="." /></th>
</xsl:template>

<xsl:template name="ydim">
  <xsl:for-each select="table[generate-id() = generate-id(key('ylabels',@ylabel)[1])]">
    <xsl:call-template name="rows"/>
  </xsl:for-each>
</xsl:template>

<xsl:template name="rows">
  <xsl:for-each select="rows/row">
  <xsl:variable name="rowid" select="position()"/>
  <tr>
   <xsl:if test="count(preceding-sibling::*) &lt; 1">
     <td rowspan="{count(../row)}"><xsl:value-of select="../../@ylabel"/></td>
   </xsl:if>
   <td><xsl:value-of select="@label"/></td>
   <xsl:for-each select="../../../table[generate-id() = generate-id(key('xlabels',@xlabel)[1])]">
     <xsl:call-template name="cols">
       <xsl:with-param name="rowid" select="$rowid"/>
     </xsl:call-template>
   </xsl:for-each>
  </tr>
  </xsl:for-each>
</xsl:template>

<xsl:template name="cols">
  <xsl:param name="rowid" select="count(rows/row)"/>
  <xsl:for-each select="rows/row[$rowid]">
    <xsl:apply-templates />
  </xsl:for-each>
</xsl:template>

<xsl:template match="col">
  <td class="num"><xsl:value-of select="."/></td>
</xsl:template>

<xsl:template match="row">
  <tr>
    <xsl:if test="count(preceding-sibling::*) &lt; 1">
      <td rowspan="{count(../row)}"><xsl:value-of select="../../@ylabel"/></td>
    </xsl:if>
   <td><xsl:value-of select="./@label"/></td>
   <xsl:apply-templates />
  </tr>
</xsl:template>

<xsl:template match="text()|node()"/>
</xsl:stylesheet>
